require_relative '../../version1_0'
require_relative '../../function'
require_relative '../../signature'
require_relative '../../semantics'

module BELParser
  module Language
    module Version1_0
      module Functions
        # Pathology: Denotes a disease or pathology process
        class Pathology
          extend Function

          SHORT       = :path
          LONG        = :pathology
          RETURN_TYPE = BELParser::Language::Version1_0::ReturnTypes::Pathology
          O_ENC       = Version1_0::ValueEncodings::Pathology
          DESCRIPTION = 'Denotes a disease or pathology process'.freeze

          def self.short
            SHORT
          end

          def self.long
            LONG
          end

          def self.return_type
            RETURN_TYPE
          end

          def self.description
            DESCRIPTION
          end

          def self.signatures
            SIGNATURES
          end

          module Signatures
            # PathologySignature
            class PathologySignature
              extend BELParser::Language::Signature

              private_class_method :new

              AST = BELParser::Language::Semantics::Builder.build do
                term(
                  function(
                    identifier(
                      function_of(Pathology))),
                  argument(
                    parameter(
                      prefix(
                        has_namespace,
                        namespace_of(:*)),
                      value(
                        has_encoding,
                        encoding_of(O_ENC)))))
              end
              private_constant :AST

              STRING_FORM = 'pathology(E:pathology)pathology'.freeze
              private_constant :STRING_FORM

              def self.semantic_ast
                AST
              end

              def self.string_form
                STRING_FORM
              end
            end
          end

          SIGNATURES = Signatures.constants.map do |const|
            Signatures.const_get(const)
          end.freeze
        end
      end
    end
  end
end
