require_relative '../../version2_0'
require_relative '../../function'
require_relative '../../signature'
require_relative '../../semantics'

module BELParser
  module Language
    module Version2_0
      module Functions
        # ToLocation: Denotes the to cellular location of the abundance.
        class ToLocation
          extend Function

          SHORT       = :toLoc
          LONG        = :toLocation
          RETURN_TYPE = BELParser::Language::Version2_0::ReturnTypes::ToLocation
          A_ENC       = Version2_0::ValueEncodings::Abundance
          DESCRIPTION = 'Denotes the to cellular location of the
          abundance.'.freeze

          def self.short
            SHORT
          end

          def self.long
            LONG
          end

          def self.return_type
            RETURN_TYPE
          end

          def self.description
            DESCRIPTION
          end

          def self.signatures
            SIGNATURES
          end

          module Signatures
            # ToLocationSignature
            class ToLocationSignature
              extend BELParser::Language::Signature

              private_class_method :new

              AST = BELParser::Language::Semantics::Builder.build do
                term(
                  function(
                    identifier(
                      function_of(ToLocation))),
                  argument(
                    parameter(
                      prefix(
                        has_namespace,
                        namespace_of(:*)),
                      value(
                        has_encoding,
                        encoding_of(A_ENC)))))
              end
              private_constant :AST

              STRING_FORM = 'toLocation(E:location)toLocation'.freeze
              private_constant :STRING_FORM

              def self.semantic_ast
                AST
              end

              def self.string_form
                STRING_FORM
              end
            end

            # ToLocationAnySignature
            class ToLocationAnySignature
              extend BELParser::Language::Signature

              private_class_method :new

              AST = BELParser::Language::Semantics::Builder.build do
                term(
                    function(
                        identifier(
                            function_of(ToLocation))),
                    argument(
                        parameter(
                            prefix(any),
                            value(any))))
              end
              private_constant :AST

              STRING_FORM = 'toLocation(Any)toLocation'.freeze
              private_constant :STRING_FORM

              def self.semantic_ast
                AST
              end

              def self.string_form
                STRING_FORM
              end
            end

          end

          SIGNATURES = Signatures.constants.map do |const|
            Signatures.const_get(const)
          end.freeze
        end
      end
    end
  end
end
